#ifdef NMZ_HASHLIBRARY
#include <openssl/sha.h>
#endif

#include <iostream>
#include "general.h"
#include "libnormaliz/nmz_hash.h"
// hash-library by Stephan Brumme
// https://create.stephan-brumme.com/hash-library/
// https://github.com/stbrumme/hash-library

namespace libnormaliz {

using namespace std;

// Compute SHA-256 hash of a string, excluding final zero
// Return hex-values as string over '0',...,'9','a',...,'f'
// return string of 32 zeros as default if hash-library is not present
string sha256str(const string& text, bool verbose) {
#ifdef NMZ_HASHLIBRARY
    unsigned char rawHash[SHA256_DIGEST_LENGTH];
    char hashstr[SHA256_DIGEST_LENGTH * 2 + 1];
    SHA256(reinterpret_cast<const unsigned char *>(text.c_str()), text.size(), rawHash);
    for (int i = 0; i < SHA256_DIGEST_LENGTH; ++i)
        sprintf(&hashstr[i * 2], "%02x", rawHash[i]);
    hashstr[SHA256_DIGEST_LENGTH * 2] = '\0';
    return hashstr;
#else
    if(verbose)
        verboseOutput() << "sha256str called but hash-library not present; "
                           "returning default value." << endl;
    // string s = "00000000000000000000000000000000"; // 32 zeros
    string s = "0";
    return s;
#endif
}

// Compute SHA-256 hash of a string, excluding final zero
// Return as vector<unsigned char> of size sha256.HashBytes (== 32)
vector<unsigned char> sha256hexvec(const string& text, bool verbose) {
#ifdef NMZ_HASHLIBRARY
    unsigned char rawHash[SHA256_DIGEST_LENGTH];
    SHA256(reinterpret_cast<const unsigned char *>(text.c_str()), text.size(), rawHash);
    int s = sizeof(rawHash)/sizeof(rawHash[0]);
    // assuming sizeof(unsigned char) == 1, then s == sha256.HashBytes == 32
    vector<unsigned char> v(rawHash, rawHash + s);
    return v;
#else
    if(verbose)
        verboseOutput() << "sha256hexvec called but hash-library not present; "
                           "returning default value." << endl;
    // vector<unsigned char> v(32, '0');
    vector<unsigned char> v(1, '0');
    // vector<unsigned char> v;
    // cout << v.size() << endl;
    return v;
#endif
}
}  // namespace libnormaliz
